<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text"/>
<xsl:strip-space elements="*"/>

<!--
	$Id: xml2pf.xsl,v 1.1.4.3 2004/10/02 20:42:54 acidos Exp $

	Converts PfPro XML configs to pf.conf format
-->

<xsl:param name="timestamp"/>
<xsl:param name="user"/>

<xsl:template match="firewall">
###
### Generated by PfPro	( http://pfpro.sourceforge.net/ )
### by <xsl:value-of select="$user"/> at <xsl:value-of select="$timestamp"/>
###

########################################################################
# BEGIN MACROS
########################################################################
<xsl:for-each select="macro">
	<xsl:value-of select="@name"/>
	<xsl:text>="</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>"&#xa;</xsl:text>
</xsl:for-each>
########################################################################
# BEGIN NORMALIZATION
########################################################################
<xsl:for-each select="scrub">
	<xsl:text>scrub </xsl:text>
	<xsl:value-of select="@direction"/>
	<xsl:text> all</xsl:text>

	<xsl:if test="@nodf">
		<xsl:text> no-df</xsl:text>
	</xsl:if>

	<xsl:if test="@maxmss">
		<xsl:text> max-mss </xsl:text>
		<xsl:value-of select="@maxmss"/>
	</xsl:if>

	<xsl:if test="@randomid">
		<xsl:text> random-id</xsl:text>
	</xsl:if>

	<xsl:text>&#xa;</xsl:text>
</xsl:for-each>
########################################################################
# BEGIN REDIRECTS
########################################################################
<xsl:for-each select="redirect">
	<xsl:text>rdr on </xsl:text>
	<xsl:value-of select="interface"/>
	<xsl:text> </xsl:text>
	<xsl:value-of select="ip"/>

	<xsl:if test="proto">
		<xsl:apply-templates select="proto"/>
	</xsl:if>

	<!-- HANDLE TO AND FROM -->
	<xsl:apply-templates select="from"/>
	<xsl:apply-templates select="to"/>

	<xsl:text> port </xsl:text>
	<xsl:value-of select="port"/>
	<xsl:text> -> </xsl:text>
	<xsl:value-of select="dest"/>
	<xsl:text> port </xsl:text>
	<xsl:value-of select="destport"/>
	<xsl:text>&#xa;</xsl:text>
</xsl:for-each>
########################################################################
# BEGIN NAT
########################################################################
<xsl:for-each select="nat">
	<xsl:text>nat on </xsl:text>
	<xsl:value-of select="interface"/>
	<xsl:text> </xsl:text>
	<xsl:value-of select="ip"/>

	<!-- HANDLE TO AND FROM -->
	<xsl:apply-templates select="from"/>
	<xsl:apply-templates select="to"/>

	<xsl:text> -> </xsl:text>
	<xsl:value-of select="via"/>
	<xsl:text>&#xa;</xsl:text>
</xsl:for-each>
########################################################################
# BEGIN FILTERING
########################################################################
<xsl:for-each select="rule">
	<xsl:sort select="@num"/>
	<!-- COMMENTS -->
	<xsl:if test="comment">
		<xsl:text>### </xsl:text>
		<xsl:value-of select="comment"/>
		<xsl:text>&#xa;</xsl:text>
	</xsl:if>
	
	<!-- PASS/BLOCK IN/OUT -->
	<xsl:value-of select="@option"/>
	<xsl:text> </xsl:text>

	<!-- DIRECTION -->
	<xsl:if test="@direction != 'all'">
		<xsl:value-of select="@direction"/>
	</xsl:if>

	<!-- LOGGING -->
	<xsl:if test="@log = 'true'">
		<xsl:text> log</xsl:text>
	</xsl:if>

	<!-- APPLY RULE QUICKLY? -->
	<xsl:if test="@quick = 'true'">
		<xsl:text> quick</xsl:text>
	</xsl:if>

	<!-- HANDLE INTERFACE SPECIFICATION -->
	<xsl:if test="interface">
		<xsl:text> on </xsl:text>
		<xsl:value-of select="interface"/>
	</xsl:if>

	<!-- INET OR INET6 -->
	<xsl:if test="ip">
		<xsl:text> </xsl:text><xsl:value-of select="@ip"/>
	</xsl:if>

	<!-- HANDLE PROTOCOLS -->
	<xsl:if test="proto">
		<xsl:apply-templates select="proto"/>
	</xsl:if>

	<!-- HANDLE TO AND FROM -->
	<xsl:apply-templates select="from"/>
	<xsl:apply-templates select="to"/>

	<!-- HANDLE PORTS -->
	<xsl:if test="port">
		<xsl:text> port </xsl:text>

		<xsl:choose>
			<xsl:when test="count(port) &gt; 1">
				<xsl:text>{ </xsl:text>

				<xsl:for-each select="port">
					<xsl:value-of select="."/>

					<xsl:if test="position() != last()">
						<xsl:text>, </xsl:text>
					</xsl:if>
				</xsl:for-each>

				<xsl:text> }</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="port"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>

	<xsl:if test="user">
		<xsl:text> user </xsl:text>

		<xsl:choose>
			<xsl:when test="count(user) &gt; 1">
				<xsl:text>{ </xsl:text>

				<xsl:for-each select="user">
					<xsl:value-of select="."/>

					<xsl:if test="position() != last()">
						<xsl:text>, </xsl:text>
					</xsl:if>
				</xsl:for-each>

				<xsl:text> }</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="user"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>

	<xsl:if test="@state != ''">
		<xsl:text> </xsl:text>
		<xsl:value-of select="@state"/>
		<xsl:text> state</xsl:text>
	</xsl:if>

	<!-- DIRECTION == ALL -->
	<xsl:if test="@direction = 'all'">
		<xsl:text> all</xsl:text>
	</xsl:if>
	
	<!-- LABEL -->
	<xsl:if test="label">
		<xsl:text> label "</xsl:text>
		<xsl:value-of select="label"/>
		<xsl:text>"</xsl:text>
	</xsl:if>

	<xsl:text>&#xa;</xsl:text>
</xsl:for-each>
</xsl:template>

<xsl:template match="proto">
	<xsl:if test="position() = 1">
		<xsl:text> </xsl:text><xsl:value-of select="name()"/><xsl:text> </xsl:text>
		<xsl:if test="count(../proto) &gt; 1">
			<xsl:text>{ </xsl:text>
		</xsl:if>
	</xsl:if>
	<xsl:value-of select="."/>
	<xsl:if test="count(../proto) &gt; 1">
		<xsl:choose>
			<xsl:when test="position() = last()">
				<xsl:text> }</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>, </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
</xsl:template>

<xsl:template match="from | to">
	<xsl:text> </xsl:text>
	<xsl:value-of select="name()"/>
	<xsl:text> </xsl:text>
	<xsl:value-of select="."/>
</xsl:template>

</xsl:stylesheet>
